
// GameList.cpp

// Implements the GameList dialog functions

#include "stdafx.h"
#include "GameList.h"
#include "BAJCheck.h"





static HFONT BoldFont;





// fwd:
BOOL CALLBACK DP_GameList(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL OnInitDialog(HWND hDlg);
BOOL OnNotify    (HWND hDlg, WPARAM wParam, LPARAM lParam);
BOOL OnCommand   (HWND hDlg, WPARAM wParam, LPARAM lParam);
BOOL OnCustomDraw(HWND hDlg, NMLVCUSTOMDRAW * iNM);
BOOL OnDblClk    (HWND hDlg, NMLISTVIEW * iNM);
BOOL OnSize      (HWND hDlg);





void ShowGameList(void)
{
	BoldFont = (HFONT)GetStockObject(ANSI_FIXED_FONT);  // TODO
	DialogBox(gInstance, MAKEINTRESOURCE(IDD_GAMELIST), gWnd, DP_GameList);
}





BOOL CALLBACK DP_GameList(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG: return OnInitDialog(hDlg);
		case WM_COMMAND:    return OnCommand   (hDlg, wParam, lParam);
		case WM_NOTIFY:     return OnNotify    (hDlg, wParam, lParam);
		case WM_SIZE:       return OnSize      (hDlg);
	}
	return FALSE;
}





static CGamePtr FindGameForParam(LPARAM lParam)
{
	for (CGamePtrs::iterator itr = gGameList.begin(); itr != gGameList.end(); ++itr)
	{
		if (atoi((*itr)->PartieID.c_str()) == lParam)
		{
			return *itr;
		}
	}
	return NULL;
}





/*
// Doesn't seem to work
static void PaintPlayers(CGamePtr iGame, NMLVCUSTOMDRAW * iNM)
{
	// Erase the background:
	HBRUSH br = CreateSolidBrush(iGame->IsMyTurn ? 0x7f7fff : 0xffffff);
	FillRect(iNM->nmcd.hdc, &(iNM->nmcd.rc), br);
	DeleteObject(br);

	SIZE siz;	
	GetTextExtentPoint32(iNM->nmcd.hdc, "My", 2, &siz);
	int y = (iNM->nmcd.rc.top + iNM->nmcd.rc.bottom + siz.cx) / 2;
	int CurX = iNM->nmcd.rc.left;
	HFONT NormalFont = (HFONT)SelectObject(iNM->nmcd.hdc, BoldFont);
	SetTextAlign(iNM->nmcd.hdc, TA_BOTTOM | TA_LEFT);
	SetTextColor(iNM->nmcd.hdc, iNM->clrText);
	for (CPlayerPtrs::iterator itr = iGame->Players.begin(); itr != iGame->Players.end(); ++itr)
	{
		if ((*itr)->IsOnTurn)
		{
			SelectObject(iNM->nmcd.hdc, BoldFont);
		}
		else
		{
			SelectObject(iNM->nmcd.hdc, NormalFont);
		}
		TextOut(iNM->nmcd.hdc, CurX, y, (*itr)->Name.c_str(), (*itr)->Name.length());
		SIZE siz;
		GetTextExtentPoint32(iNM->nmcd.hdc, (*itr)->Name.c_str(), (*itr)->Name.length(), &siz);
		CurX += siz.cx;
	}  // for itr - iGame->Players[]
	SelectObject(iNM->nmcd.hdc, NormalFont);
}
*/





static int CALLBACK SortLVItems(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	// Find the game objects:
	CGamePtr Game1 = NULL;
	CGamePtr Game2 = NULL;
	for (CGamePtrs::iterator itr = gGameList.begin(); itr != gGameList.end(); ++itr)
	{
		if (atoi((*itr)->PartieID.c_str()) == lParam1)
		{
			Game1 = *itr;
		}
		if (atoi((*itr)->PartieID.c_str()) == lParam2)
		{
			Game2 = *itr;
		}
	}  // for itr - gGameList[]
	if ((Game1 == NULL) || (Game2 == NULL))
	{
		return 0;
	}
	
	if (Game1->IsMyTurn && !Game2->IsMyTurn)
	{
		return -1;
	}
	if (Game2->IsMyTurn && !Game1->IsMyTurn)
	{
		return 1;
	}
	
	// Compare game types:
	int c = Game1->Type.compare(Game2->Type);
	if (c != 0)
	{
		return c;
	}
	
	// Compare PartieIDs:
	return lParam1 - lParam2;
}





static BOOL OnInitDialog(HWND hDlg)
{
	HWND lv = GetDlgItem(hDlg, IDC_LV_GAMES);
	ListView_SetExtendedListViewStyle(lv, LVS_EX_FULLROWSELECT);
	
	// Fill in columns:
	LV_COLUMN lc;
	ZeroMemory(&lc, sizeof(lc));
	lc.mask = LVCF_TEXT | LVCF_WIDTH;
	lc.pszText = TEXT("Game ID");
	lc.cx = 60;
	ListView_InsertColumn(lv, 0, &lc);

	lc.mask = LVCF_TEXT | LVCF_WIDTH;
	lc.pszText = TEXT("Game Type");
	lc.cx = 150;
	ListView_InsertColumn(lv, 1, &lc);
	
	lc.mask = LVCF_TEXT | LVCF_WIDTH;
	lc.pszText = TEXT("Players");
	lc.cx = 250;
	ListView_InsertColumn(lv, 2, &lc);

	// Fill in games:
	for (CGamePtrs::iterator itr = gGameList.begin(); itr != gGameList.end(); ++itr)
	{
		LV_ITEMA li;
		ZeroMemory(&li, sizeof(li));
		li.mask = LVIF_PARAM | LVIF_TEXT;
		li.pszText = (LPSTR)((*itr)->PartieID.c_str());
		li.lParam = atoi((*itr)->PartieID.c_str());
		li.iItem = ListView_InsertItem(lv, &li);
		
		li.mask = LVIF_TEXT;
		li.pszText = (LPSTR)((*itr)->Type.c_str());
		li.iSubItem = 1;
		ListView_SetItem(lv, &li);
		
		std::string Players = (*itr)->GetPlayersString();
		li.pszText = (LPSTR)(Players.c_str());
		li.iSubItem = 2;
		ListView_SetItem(lv, &li);
	}
	ListView_SortItems(lv, SortLVItems, &gGameList);

	SendMessage(hDlg, WM_SETICON, ICON_SMALL, (LPARAM)gIcoSmall);
	SendMessage(hDlg, WM_SETICON, ICON_BIG,   (LPARAM)gIcoLarge);
	
	OnSize(hDlg);
		
	return TRUE;
}




static BOOL OnCommand(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	switch (LOWORD(wParam))
	{
		case IDCANCEL:
		{
			EndDialog(hDlg, 0);
			return TRUE;
		}
	}
	return FALSE;
}





static BOOL OnNotify(HWND hDlg, WPARAM wParam, LPARAM lParam)
{
	NMHDR * nmh = (NMHDR *)lParam;
	switch (nmh->code)
	{
		case NM_CUSTOMDRAW: return OnCustomDraw(hDlg, (NMLVCUSTOMDRAW *)nmh);
		case NM_DBLCLK:     return OnDblClk(hDlg, (NMLISTVIEW *)nmh);
	}
	return FALSE;
}





static BOOL OnCustomDraw(HWND hDlg, NMLVCUSTOMDRAW * iNM)
{
	switch (iNM->nmcd.dwDrawStage)
	{
		case CDDS_PREPAINT:
		{
			SetWindowLongPtr(hDlg, DWLP_MSGRESULT, CDRF_NOTIFYITEMDRAW);
			return TRUE;
		}
		
		case CDDS_ITEMPREPAINT:
		{
			CGamePtr g = FindGameForParam(iNM->nmcd.lItemlParam);
			if (g == NULL)
			{
				SetWindowLongPtr(hDlg, DWLP_MSGRESULT, CDRF_DODEFAULT);
				return TRUE;
			}
			if (g->IsMyTurn)
			{
				iNM->clrTextBk = 0x7f7fff;
			}
			// Doesn't seem to work: SetWindowLongPtr(hDlg, DWLP_MSGRESULT, CDRF_DODEFAULT | CDRF_NOTIFYSUBITEMDRAW);
			SetWindowLongPtr(hDlg, DWLP_MSGRESULT, CDRF_DODEFAULT);
			return TRUE;
		}
		
		/*
		// Doesn't seem to work
		case CDDS_ITEMPREPAINT | CDDS_SUBITEM:
		{
			if (iNM->iSubItem != 2)
			{
				SetWindowLongPtr(hDlg, DWLP_MSGRESULT, CDRF_DODEFAULT);
				return TRUE;
			}
			CGamePtr g = FindGameForParam(iNM->nmcd.lItemlParam);
			if (g == NULL)
			{
				SetWindowLongPtr(hDlg, DWLP_MSGRESULT, CDRF_DODEFAULT);
				return TRUE;
			}
			
			PaintPlayers(g, iNM);
			
			SetWindowLongPtr(hDlg, DWLP_MSGRESULT, CDRF_SKIPDEFAULT);
			return TRUE;
		}
		*/
	}
	return TRUE;
}




BOOL OnDblClk(HWND hDlg, NMLISTVIEW * iNM)
{
	LV_ITEM li;
	ZeroMemory(&li, sizeof(li));
	li.mask = LVIF_PARAM;
	li.iItem = iNM->iItem;
	if (!ListView_GetItem(GetDlgItem(hDlg, IDC_LV_GAMES), &li))
	{
		return TRUE;
	}
	CGamePtr g = FindGameForParam(li.lParam);
	if (g == NULL)
	{
		return TRUE;
	}
	g->OpenInBrowser();
	return TRUE;
}




BOOL OnSize(HWND hDlg)
{
	HWND tl = GetDlgItem(hDlg, IDCANCEL);
	HWND lv = GetDlgItem(hDlg, IDC_LV_GAMES);
	RECT rc;
	GetWindowRect(tl, &rc);
	int tlWid = rc.right - rc.left;
	int tlHei = rc.bottom - rc.top;
	GetClientRect(hDlg, &rc);
	int wid = rc.right - rc.left;
	int hei = rc.bottom - rc.top;
	MoveWindow(tl, wid - tlWid - 4, hei - tlHei - 4, tlWid, tlHei, TRUE);
	MoveWindow(lv, 4, 4, wid - 4, hei - tlHei - 12, TRUE);
	return TRUE;
}




